@@ -15,11 +15,17 @@
 
 from __future__ import print_function
 
+import errno
+import os
+import re
+import signal
 import sys
+import time
 import logging
 from logging.config import fileConfig
 
-from optparse import OptionParser
+from subprocess import Popen
+from optparse import OptionParser, OptionValueError
 
 from mapproxy.config.loader import load_configuration, ConfigurationError
 from mapproxy.seed.config import load_seed_tasks_conf
@@ -29,6 +35,9 @@ from mapproxy.seed.util import (format_seed_task, format_cleanup_task,
     ProgressLog, ProgressStore)
 from mapproxy.seed.cachelock import CacheLocker
 
+SECONDS_PER_DAY = 60 * 60 * 24
+SECONDS_PER_MINUTE = 60
+
 def setup_logging(logging_conf=None):
     if logging_conf is not None:
         fileConfig(logging_conf, {'here': './'})
@@ -43,12 +52,41 @@ def setup_logging(logging_conf=None):
     ch.setFormatter(formatter)
     mapproxy_log.addHandler(ch)
 
+
+def check_duration(option, opt, value, parser):
+    try:
+        setattr(parser.values, option.dest, parse_duration(value))
+    except ValueError:
+        raise OptionValueError(
+            "option %s: invalid duration value: %r, expected (10s, 15m, 0.5h, 3d, etc)"
+            % (opt, value),
+        )
+
+
+def parse_duration(string):
+    match = re.match(r'^(\d*.?\d+)(s|m|h|d)', string)
+    if not match:
+        raise ValueError('invalid duration, not in format: 10s, 0.5h, etc.')
+    duration = float(match.group(1))
+    unit = match.group(2)
+    if unit == 's':
+        return duration
+    duration *= 60
+    if unit == 'm':
+        return duration
+    duration *= 60
+    if unit == 'h':
+        return duration
+    duration *= 24
+    return duration
+
+
 class SeedScript(object):
     usage = "usage: %prog [options] seed_conf"
     parser = OptionParser(usage)
     parser.add_option("-q", "--quiet",
                       action="count", dest="quiet", default=0,
-                      help="don't print status messages to stdout")
+                      help="reduce number of messages to stdout, repeat to disable progress output")
     parser.add_option("-s", "--seed-conf",
                       dest="seed_file", default=None,
                       help="seed configuration")
@@ -97,6 +135,19 @@ class SeedScript(object):
                       default=None,
                       help="filename for storing the seed progress (for --continue option)")
 
+    parser.add_option("--duration", dest="duration",
+                      help="stop seeding after (120s, 15m, 4h, 0.5d, etc)",
+                      type=str, action="callback", callback=check_duration)
+
+    parser.add_option("--reseed-file", dest="reseed_file",
+                      help="start of last re-seed", metavar="FILE",
+                      default=None)
+    parser.add_option("--reseed-interval", dest="reseed_interval",
+                      help="only start seeding if --reseed-file is older then --reseed-interval",
+                      metavar="DURATION",
+                      type=str, action="callback", callback=check_duration,
+                      default=None)
+
     parser.add_option("--log-config", dest='logging_conf', default=None,
                       help="logging configuration")
 
@@ -118,6 +169,10 @@ class SeedScript(object):
 
         setup_logging(options.logging_conf)
 
+        if options.duration:
+            # calls with --duration are handled in call_with_duration
+            sys.exit(self.call_with_duration(options, args))
+
         try:
             mapproxy_conf = load_configuration(options.conf_file, seed=True)
         except ConfigurationError as ex:
@@ -133,6 +188,29 @@ class SeedScript(object):
             # disable verbose output for non-ttys
             options.quiet = 1
 
+        progress = None
+        if options.continue_seed or options.progress_file:
+            if not options.progress_file:
+                options.progress_file = '.mapproxy_seed_progress'
+            progress = ProgressStore(options.progress_file,
+                                     continue_seed=options.continue_seed)
+
+        if options.reseed_file:
+            if not os.path.exists(options.reseed_file):
+                # create --reseed-file if missing
+                with open(options.reseed_file, 'w'):
+                    pass
+            else:
+                if progress and not os.path.exists(options.progress_file):
+                    # we have an existing --reseed-file but no --progress-file
+                    # meaning the last seed call was completed
+                    if options.reseed_interval and (
+                        os.path.getmtime(options.reseed_file) > (time.time() - options.reseed_interval)
+                    ):
+                        print("no need for re-seeding")
+                        sys.exit(1)
+                    os.utime(options.reseed_file, (time.time(), time.time()))
+
         with mapproxy_conf:
             try:
                 seed_conf = load_seed_tasks_conf(options.seed_file, mapproxy_conf)
@@ -152,15 +230,6 @@ class SeedScript(object):
                     print(format_cleanup_task(task))
                 return 0
 
-            progress = None
-            if options.continue_seed or options.progress_file:
-                if options.progress_file:
-                    progress_file = options.progress_file
-                else:
-                    progress_file = '.mapproxy_seed_progress'
-                progress = ProgressStore(progress_file,
-                    continue_seed=options.continue_seed)
-
             try:
                 if options.interactive:
                     seed_tasks, cleanup_tasks = self.interactive(seed_tasks, cleanup_tasks)
@@ -178,7 +247,8 @@ class SeedScript(object):
                     print('========== Cleanup tasks ==========')
                     print('Start cleanup process (%d task%s)' % (
                         len(cleanup_tasks), 's' if len(cleanup_tasks) > 1 else ''))
-                    logger = ProgressLog(verbose=options.quiet==0, silent=options.quiet>=2)
+                    logger = ProgressLog(verbose=options.quiet==0, silent=options.quiet>=2,
+                        progress_store=progress)
                     cleanup(cleanup_tasks, verbose=options.quiet==0, dry_run=options.dry_run,
                             concurrency=options.concurrency, progress_logger=logger,
                             skip_geoms_for_last_levels=options.geom_levels)
@@ -225,6 +295,48 @@ class SeedScript(object):
 
         return seed_names, cleanup_names
 
+    def call_with_duration(self, options, args):
+        # --duration is implemented by calling mapproxy-seed again in a separate
+        # process (but without --duration) and terminating that process
+        # after --duration
+
+        argv = sys.argv[:]
+        for i, arg in enumerate(sys.argv):
+            if arg == '--duration':
+                argv = sys.argv[:i] + sys.argv[i+2:]
+                break
+            elif arg.startswith('--duration='):
+                argv = sys.argv[:i] + sys.argv[i+1:]
+                break
+
+        # call mapproxy-seed again, poll status, terminate after --duration
+        cmd = Popen(args=argv)
+        start = time.time()
+        while True:
+            if (time.time() - start) > options.duration:
+                try:
+                    cmd.send_signal(signal.SIGINT)
+                    # try to stop with sigint
+                    # send sigterm after 10 seconds
+                    for _ in range(10):
+                        time.sleep(1)
+                        if cmd.poll() is not None:
+                            break
+                    else:
+                        cmd.terminate()
+                except OSError as ex:
+                    if ex.errno != errno.ESRCH:  # no such process
+                        raise
+                return 0
+            if cmd.poll() is not None:
+                return cmd.returncode
+            try:
+                time.sleep(1)
+            except KeyboardInterrupt:
+                # force termination
+                start = 0
+
+
     def interactive(self, seed_tasks, cleanup_tasks):
         selected_seed_tasks = []
         print('========== Select seeding tasks ==========')
@@ -264,5 +376,6 @@ def split_comma_seperated_option(option):
             result.extend(args.split(','))
     return result
 
+
 if __name__ == '__main__':
     main()
