@@ -323,6 +323,11 @@ class TestTileManagerWMSSource(object):
              ((0.0, -90.0, 180.0, 90.0), (512, 512), SRS(4326))])
 
 
+class TestTileManagerWMSSourceConcurrent(TestTileManagerWMSSource):
+    def setup(self):
+        TestTileManagerWMSSource.setup(self)
+        self.tile_mgr.concurrent_tile_creators = 2
+
 class TestTileManagerWMSSourceMinimalMetaRequests(object):
     def setup(self):
         self.file_cache = MockFileCache('/dev/null', 'png')
@@ -482,6 +487,65 @@ class TestTileManagerMultipleSourcesWithMetaTiles(object):
 
         assert self.tile_mgr.meta_grid is None
 
+
+class TestTileManagerBulkMetaTiles(object):
+    def setup(self):
+        self.file_cache = MockFileCache('/dev/null', 'png')
+        self.grid = TileGrid(SRS(4326), bbox=[-180, -90, 180, 90], origin='ul')
+        self.source_base = SolidColorMockSource(color='#ff0000')
+        self.source_base.supports_meta_tiles = False
+        self.source_overlay = MockSource()
+        self.source_overlay.supports_meta_tiles = False
+        self.locker = TileLocker(tmp_lock_dir, 10, "id")
+        self.tile_mgr = TileManager(self.grid, self.file_cache,
+            [self.source_base, self.source_overlay], 'png',
+            meta_size=[2, 2], meta_buffer=0,
+            locker=self.locker,
+            bulk_meta_tiles=True,
+        )
+
+    def test_bulk_get(self):
+        tiles = self.tile_mgr.creator().create_tiles([Tile((0, 0, 2))])
+        eq_(len(tiles), 2*2)
+        eq_(self.file_cache.stored_tiles, set([(0, 0, 2), (1, 0, 2), (0, 1, 2), (1, 1, 2)]))
+        for requested in [self.source_base.requested, self.source_overlay.requested]:
+            eq_(set(requested), set([
+                ((-180.0, 0.0, -90.0, 90.0), (256, 256), SRS(4326)),
+                ((-90.0, 0.0, 0.0, 90.0), (256, 256), SRS(4326)),
+                ((-180.0, -90.0, -90.0, 0.0), (256, 256), SRS(4326)),
+                ((-90.0, -90.0, 0.0, 0.0), (256, 256), SRS(4326)),
+            ]))
+
+    def test_bulk_get_error(self):
+        self.tile_mgr.sources = [self.source_base, ErrorSource()]
+        try:
+            self.tile_mgr.creator().create_tiles([Tile((0, 0, 2))])
+        except Exception as ex:
+            eq_(ex.args[0], "source error")
+
+    def test_bulk_get_multiple_meta_tiles(self):
+        tiles = self.tile_mgr.creator().create_tiles([Tile((1, 0, 2)), Tile((2, 0, 2))])
+        eq_(len(tiles), 2*2*2)
+        eq_(self.file_cache.stored_tiles, set([
+            (0, 0, 2), (1, 0, 2), (0, 1, 2), (1, 1, 2),
+            (2, 0, 2), (3, 0, 2), (2, 1, 2), (3, 1, 2),
+        ]))
+
+class ErrorSource(MapLayer):
+    def __init__(self, *args):
+        MapLayer.__init__(self, *args)
+        self.requested = []
+
+    def get_map(self, query):
+        self.requested.append((query.bbox, query.size, query.srs))
+        raise Exception("source error")
+
+class TestTileManagerBulkMetaTilesConcurrent(TestTileManagerBulkMetaTiles):
+    def setup(self):
+        TestTileManagerBulkMetaTiles.setup(self)
+        self.tile_mgr.concurrent_tile_creators = 2
+
+
 default_image_opts = ImageOptions(resampling='bicubic')
 
 class TestCacheMapLayer(object):
